from fabric.api import task, env

from hemp.utils import print_err, print_info


@task(name='on')
def on(env_name=None):
    if env_name is None:
        print_err('Please, provide environment name, like, on:staging')
    if not env.hemp:
        print_err('No hemp entry in environment')
    if not env.hemp['environments']:
        print_err('No hemp environments defined')
    if env_name not in env.hemp['environments']:
        print_err('Environment not defined: {0}'.format(env_name))
    environment = env.hemp['environments'][env_name]
    if 'roles' not in environment:
        print_err('Roles are not defined for this environment')

    all_hosts = []

    for role, hosts in environment['roles'].items():
        all_hosts += hosts
        if role not in env.roledefs:
            env.roledefs[role] = hosts
        else:
            env.roledefs[role] = list(set(env.roledefs[role] + hosts))

    all_hosts = list(set(all_hosts))
    env.hosts = list(set(all_hosts + env.hosts))

    print_info('Loaded hosts and roles for {0}'.format(env_name))


@task
def staging():
    on('staging')


@task
def production():
    on('production')


@task
def development():
    on('development')



from os import getcwd
from os.path import expanduser, join, isfile

import yaml
from fabric.state import env

_default_locations = [
    expanduser('~'),
    getcwd()
]


def discover_hempfiles(extra_locations=None):
    locations = _default_locations
    if extra_locations is not None:
        locations = extra_locations + _default_locations
    files = []
    for directory in locations:
        hemp_file = join(directory, 'hemp.yml')
        if isfile(hemp_file):
            files.append(hemp_file)
    return files


def parse_hempfiles(file_paths=None):
    if file_paths is None:
        file_paths = discover_hempfiles()
    config = {}
    for file_path in file_paths:
        stream = open(file_path, 'r')
        entries = yaml.load(stream)
        config = _deep_merge(config, entries)
    return config


def load_hempfiles(file_paths=None):
    if 'hemp' in env:
        return
    config = parse_hempfiles(file_paths)
    for option, value in config.items():
        setattr(env, option, value)


def _deep_merge(source, destination):
    for key, value in source.items():
        if isinstance(value, dict):
            node = destination.setdefault(key, {})
            _deep_merge(value, node)
        else:
            destination[key] = value
    return destination



from fabric.utils import error, puts


def print_err(message, func=None, exception=None, stdout=None, stderr=None):
    error('[Hemp] ' + message, func, exception, stdout, stderr)


def print_info(text, show_prefix=None, end="\n", flush=True):
    puts('[Hemp] ' + text, show_prefix, end, flush)



from fabric import main as fabric_main
from fabric import state
from fabric.main import load_tasks_from_module

from hemp import api
from hemp.hempfile import load_hempfiles
from hemp.utils import print_info


def main(fabfile_locations=None, file_paths=None):
    if fabfile_locations is None:
        fabfile_locations = ['~/fabfile.py']
        print_info('Added $HOME to fabfile locations')

    docstring, new_style, classic, default = load_tasks_from_module(api)
    tasks = new_style if state.env.new_style_tasks else classic
    state.commands.update(tasks)

    print_info('Forwarding execution to Fabric')

    _load_settings_original = fabric_main.load_settings

    def hemp_load_settings(path):
        print_info('Loading hempfiles')
        load_hempfiles(file_paths)
        return _load_settings_original(path)

    fabric_main.load_settings = hemp_load_settings
    fabric_main.main(fabfile_locations)



#!/usr/bin/env python

from __future__ import with_statement

from setuptools import setup, find_packages

setup(
    name='Hemp',
    version="0.0.4",
    description='Tools for Fabric',
    long_description="Tools for Fabric",
    author='Addvilz',
    author_email='mrtreinis@gmail.com',
    url='https://github.com/Addvilz/hemp',
    download_url='https://github.com/Addvilz/hemp',
    license='Apache 2.0',
    platforms='UNIX',
    packages=find_packages(),
    install_requires=[
        "fabric>=1.12",
        "pyyaml>=3.12"
    ],
    entry_points={
        'console_scripts': [
            'hemp = hemp.main:main',
        ]
    },
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development',
        'Topic :: System :: Software Distribution',
        'Topic :: System :: Systems Administration',
    ],
)
